package geniusweb.protocol;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;

import geniusweb.protocol.session.SessionState;
import geniusweb.protocol.tournament.TournamentState;

import com.fasterxml.jackson.annotation.JsonTypeInfo;

/**
 * The current state of the tournament. Must be serializabl;e so that it can be
 * restarted if a crash occurs.
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE, isGetterVisibility = Visibility.NONE)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.WRAPPER_OBJECT)
@JsonSubTypes({ @Type(value = SessionState.class),
		@Type(value = TournamentState.class) })
public interface NegoState {

	/**
	 * @return the settings that were used to create this Nego
	 */
	NegoSettings getSettings();

	/**
	 * @param currentTimeMs the current time in ms since 1970, see
	 *                      {@link System#currentTimeMillis()}
	 * @return true iff this is the final state. A state can be final because
	 *         the protocol decided so, eg a deal was achieved, the deadline was
	 *         reached or someone made a protocol error. If true, no more state
	 *         changes can occur, including no more protocol errors.
	 */
	boolean isFinal(long currentTimeMs);

	/**
	 * 
	 * @return a fatal error occured during the tournament. returns null if no
	 *         errors occured.
	 */
	ProtocolException getError();
}
